/*
 * main.c
 * 
 * Copyright 2013 Stefan Mavrodiev <support@olimex.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 */


#define _BSD_SOURCE


#include <stdio.h>
#include <stdint.h>
#include <unistd.h>
#include <math.h>
#include <getopt.h>
#include <stdlib.h>
#include <string.h>

#include "../inc/i2c.h"
#include "../inc/color.h"
#include "../inc/HMC5883.h"

#define elif else if


static char program[10];

void printHelp() {
    printf("\
\nusage: %s [PARAMS] \
\n\
\nParameters\
\n--address -a\t-\tThe address of the board (default 0x1E)\
\n--i2c\t-i\t-\tThe name of the I2C-bus (default \"\\dev\\i2c-2\"\
\n--verbose\t-\tEnable verbose mode\
\n--high-speed\t-\tEnable for high-speed I2C communication\
\n\
\n Device settings\
\n----------------\
\n--operation-mode\
\n\t-M\t-\tSet mode of operation. Continuous, Single or Idle\
\n--gain\
\n\t-g\t-\tSet gain. Use -g --list for more information.\
\n--measurement-mode\
\n\t-m\t-\tSet measurement mode. Use -m --list for available modes\
\n--output-rate\
\n\t-o\t-\tDefine data output rate. Use -o --list for full list\
\n", program);
}

int main(int argc, char **argv) {
    int c;
    static int hs = 0;
    static int verbose = 0;
    static int continuous = 0;
    
    int fd;
    float radians;
    
    unsigned char address = 0x1E;
    char i2c[] = "/dev/i2c-2";
    
    /* Some constants needed for calculation */
    
    strcpy(program, argv[0]);
    
    /* Process all parameters */
    while (1) {
        static struct option long_options[] = {
            /* Parameters that set flags */
            {"verbose", no_argument, &verbose, 1},
            {"high-speed", no_argument, &hs, 1},
            
            
            /* System parameters */
            {"address", required_argument, 0, 'a'},
            {"i2c", required_argument, 0, 'i'},
             {"help", no_argument, 0, 'h'},
            
            /* Module specific parameters */
            {"operation-mode", required_argument, 0, 'M'},            
            {"gain", required_argument, 0, 'g'},
            {"measurement-mode", required_argument, 0, 'm'},
            {"output-rate", required_argument, 0, 'o'},
            {0, 0, 0, 0}
        };

        int option_index = 0;

        c = getopt_long(argc, argv, "a:i:h:M:g:m:o:", long_options, &option_index);

        if (c == -1)
            break;

        switch (c) {
            case 0:
                break;
            case 'o':
                /* Check for list */
                if(!strcmp(optarg, "--list"))
                    list_output_rate();
              
                /* Check if input is valid */
                elif(!strcmp(optarg, "0.75")){
                    regs[CONF_REG_A].current_value &= ~(OR_RES);
                    regs[CONF_REG_A].current_value |= OR_0075;
                } elif(!strcmp(optarg, "1.5")){
                    regs[CONF_REG_A].current_value &= ~(OR_RES);
                    regs[CONF_REG_A].current_value |= OR_0150;
                } elif(!strcmp(optarg, "3")){
                    regs[CONF_REG_A].current_value &= ~(OR_RES);
                    regs[CONF_REG_A].current_value |= OR_0300;
                } elif(!strcmp(optarg, "7.5")){
                    regs[CONF_REG_A].current_value &= ~(OR_RES);
                    regs[CONF_REG_A].current_value |= OR_0750;
                } elif(!strcmp(optarg, "15")){
                    regs[CONF_REG_A].current_value &= ~(OR_RES);
                    regs[CONF_REG_A].current_value |= OR_1500;
                } elif(!strcmp(optarg, "30")){
                    regs[CONF_REG_A].current_value &= ~(OR_RES);
                    regs[CONF_REG_A].current_value |= OR_3000;
                } elif(!strcmp(optarg, "75")){
                    regs[CONF_REG_A].current_value &= ~(OR_RES);
                    regs[CONF_REG_A].current_value |= OR_7500;
                } else {
                    printf("Invalid parameter \"%s\"!\n", optarg);
                    list_output_rate();
                    exit(EXIT_FAILURE);
                }
                break;
                
            case 'm':
                /* Check for list */
                if(!strcmp(optarg, "--list"))
                    list_measurement_mode();
                
                /* Check if input is valid */
                elif(!strcmp(optarg, "normal")){
                    regs[CONF_REG_A].current_value &= ~(MM_RES);
                    regs[CONF_REG_A].current_value |= MM_NORMAL;
                } elif(!strcmp(optarg, "positive")){
                    regs[CONF_REG_A].current_value &= ~(MM_RES);
                    regs[CONF_REG_A].current_value |= MM_POS;
                } elif(!strcmp(optarg, "negative")){
                    regs[CONF_REG_A].current_value &= ~(MM_RES);
                    regs[CONF_REG_A].current_value |= MM_NEG;
                } else {
                    printf("Invalid parameter \"%s\"!\n", optarg);
                    list_measurement_mode();
                    exit(EXIT_FAILURE);
                }
                break;
            case 'g':
                /* Check for list */
                if(!strcmp(optarg, "--list"))
                    list_gain();
                
                /* Check if input is valid */
                elif(!strcmp(optarg, "1370")){
                    regs[CONF_REG_B].current_value &= ~(GN_230);
                    regs[CONF_REG_B].current_value |= GN_1370;
                    resolution = 0.73;
                } elif(!strcmp(optarg, "1090")){
                    regs[CONF_REG_B].current_value &= ~(GN_230);
                    regs[CONF_REG_B].current_value |= GN_1090;
                    resolution = 0.92;
                } elif(!strcmp(optarg, "820")){
                    regs[CONF_REG_B].current_value &= ~(GN_230);
                    regs[CONF_REG_B].current_value |= GN_820;
                    resolution = 1.22;
                } elif(!strcmp(optarg, "660")){
                    regs[CONF_REG_B].current_value &= ~(GN_230);
                    regs[CONF_REG_B].current_value |= GN_660;
                    resolution = 1.52;
                } elif(!strcmp(optarg, "440")){
                    regs[CONF_REG_B].current_value &= ~(GN_230);
                    regs[CONF_REG_B].current_value |= GN_440;
                    resolution = 2.27;
                } elif(!strcmp(optarg, "390")){
                    regs[CONF_REG_B].current_value &= ~(GN_230);
                    regs[CONF_REG_B].current_value |= GN_390;
                    resolution = 2.56;
                } elif(!strcmp(optarg, "330")){
                    regs[CONF_REG_B].current_value &= ~(GN_230);
                    regs[CONF_REG_B].current_value |= GN_330;
                    resolution = 3.03;
                } elif(!strcmp(optarg, "230")){
                    regs[CONF_REG_B].current_value |= GN_230;
                    resolution = 4.35;
                } else {
                    printf("Invalid parameter \"%s\"!\n", optarg);
                    list_gain();
                    exit(EXIT_FAILURE);
                }
                break;
            case 'M':
                /* Check for list */
                if(!strcmp(optarg, "--list"))
                    list_operating_mode();
                
                /* Check if input is valid */
                elif(!strcmp(optarg, "continuous")){
                    regs[MOD_REG].current_value &= ~(MD_IDLE);
                    regs[MOD_REG].current_value |= MD_CONTINOUS;
                    continuous = 1;
                } elif(!strcmp(optarg, "single")){
                    regs[MOD_REG].current_value &= ~(MD_IDLE);
                    regs[MOD_REG].current_value |= MD_SINGLE;
                } elif(!strcmp(optarg, "idle")){
                    regs[MOD_REG].current_value |= MD_IDLE;
                } else {
                    printf("Invalid parameter \"%s\"!\n", optarg);
                    list_operating_mode();
                    exit(EXIT_FAILURE);
                }
                break;
            case 'a':
                address = strtol(optarg, NULL, 0);
                /* Check if address is valid */
                if(address == 0 || address == 0xFF){
                    printf("Invalid address \"%s\"!\n", optarg);
                    exit(EXIT_FAILURE);
                }                
                break;
            case 'h':
                printHelp();
                exit(EXIT_SUCCESS);
                break;
            default:
                break;
            case '?':
                printf("Unknown parameter! Try --help\n");
                return -1;
                break;
        }
    }
    /* Check for high speed */
    if(hs)
        regs[MOD_REG].current_value |= HIGH_SPEED;
    else
        regs[MOD_REG].current_value &= ~HIGH_SPEED;
    
    debug = verbose;
    
    
    
    InitializeModule(&fd, i2c, address);
    
    do{
        radians = GetHeading(&fd);
        printf("%.3f\n", (radians * 180/M_PI));
    }while(continuous);
    return 0;
}
